@@ -1,4 +1,4 @@
-# Copyright 2004-2011 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2012 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -152,11 +152,11 @@ class PyCode(object):
         if isinstance(code, renpy.python.ast.AST): #@UndefinedVariable
             code = renpy.python.ast.dump(code) #@UndefinedVariable
         
-        self.hash = chr(renpy.bytecode_version) + md5.md5(code.encode("utf-8")).digest()
+        self.hash = chr(renpy.bytecode_version) + md5.md5(repr(self.location) + code.encode("utf-8")).digest()
         return self.hash
             
             
-def chain_block(block, next):
+def chain_block(block, next): #@ReservedAssignment
     """
     This is called to chain together all of the nodes in a block. Node
     n is chained with node n+1, while the last node is chained with
@@ -182,7 +182,7 @@ class Scry(object):
     def __getattr__(self, name):
         return None
 
-    def next(self):
+    def next(self): #@ReservedAssignment
         if self._next is None:
             return None
         else:
@@ -256,7 +256,7 @@ class Node(object):
 
         return None
         
-    def chain(self, next):
+    def chain(self, next): #@ReservedAssignment
         """
         This is called with the Node node that should be followed after
         executing this node, and all nodes that this node
@@ -475,7 +475,7 @@ class Init(Node):
     # We handle chaining specially. We want to chain together the nodes in
     # the block, but we want that chain to end in None, and we also want
     # this node to just continue on to the next node in normal execution.
-    def chain(self, next):
+    def chain(self, next): #@ReservedAssignment
         self.next = next
 
         chain_block(self.block, None)
@@ -517,7 +517,7 @@ class Label(Node):
     def get_children(self):
         return self.block 
 
-    def chain(self, next):
+    def chain(self, next): #@ReservedAssignment
 
         if self.block:
             self.next = self.block[0]
@@ -627,7 +627,15 @@ class Python(Node):
 
     def execute(self):
         next_node(self.next)
-        renpy.python.py_exec_bytecode(self.code.bytecode, self.hide)
+        
+        try:        
+            renpy.python.py_exec_bytecode(self.code.bytecode, self.hide)
+        finally:
+            
+            if not renpy.game.context().init_phase:
+                for i in renpy.config.python_callbacks:
+                    i()
+                    
 
     def scry(self):
         rv = Node.scry(self)
@@ -1105,7 +1113,7 @@ class Return(Node):
         return (Return, )
 
     # We don't care what the next node is.
-    def chain(self, next):
+    def chain(self, next): #@ReservedAssignment
         self.next = None
         return
 
@@ -1141,7 +1149,7 @@ class Menu(Node):
         'with_',
         ]
 
-    def __init__(self, loc, items, set, with_):
+    def __init__(self, loc, items, set, with_): #@ReservedAssignment
         super(Menu, self).__init__(loc)
 
         self.items = items
@@ -1161,7 +1169,7 @@ class Menu(Node):
         return rv
 
     # Blocks of statements in a choice continue after the menu.
-    def chain(self, next):
+    def chain(self, next): #@ReservedAssignment
 
         self.next = next
 
@@ -1246,7 +1254,7 @@ class Jump(Node):
         return (Jump, self.target, self.expression)
 
     # We don't care what our next node is.
-    def chain(self, next):
+    def chain(self, next): #@ReservedAssignment
         self.next = None
         return
 
@@ -1309,7 +1317,7 @@ class While(Node):
     def get_children(self):
         return self.block
 
-    def chain(self, next):
+    def chain(self, next): #@ReservedAssignment
         self.next = next
         chain_block(self.block, self)
 
@@ -1352,7 +1360,7 @@ class If(Node):
 
         return rv
 
-    def chain(self, next):
+    def chain(self, next): #@ReservedAssignment
         self.next = next
 
         for _condition, block in self.entries:
@@ -1380,16 +1388,21 @@ class If(Node):
 
 class UserStatement(Node):
 
-    __slots__ = [ 'line', 'parsed' ]
+    __slots__ = [ 'line', 'parsed', 'block' ]
 
-    def __init__(self, loc, line):
+    def __setstate__(self, state):
+        self.block = [ ]
+        Node.__setstate__(self, state)
+
+    def __init__(self, loc, line, block):
 
         super(UserStatement, self).__init__(loc)
         self.line = line
+        self.block = block
         self.parsed = None
 
         # Do not store the parse quite yet.
-        renpy.statements.parse(self, self.line)
+        renpy.statements.parse(self, self.line, self.block)
         
     def diff_info(self):
         return (UserStatement, self.line)
@@ -1407,7 +1420,7 @@ class UserStatement(Node):
         
         parsed = self.parsed        
         if parsed is None:
-            parsed = renpy.statements.parse(self, self.line)
+            parsed = renpy.statements.parse(self, self.line, self.block)
             self.parsed = parsed
 
         renpy.statements.call(method, parsed, *args, **kwargs)
