@@ -1,4 +1,4 @@
-# Copyright 2004-2011 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2012 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -100,13 +100,20 @@ def compile_event(key, keydown):
         rv += " and not (ev.mod & %d)" % pygame.KMOD_SHIFT
 
     if len(part) == 1:
-        if len(part[0]) != 1:
-            raise Exception("Invalid key specifier %s" % key)
+        if len(part[0]) != 1:            
+            if renpy.config.developer:
+                raise Exception("Invalid key specifier %s" % key)
+            else:
+                return "(False)"
+
         rv += " and ev.unicode == %r)" % part[0]        
 
     else:
         if part[0] != "K":
-            raise Exception("Invalid key specifier %s" % key)
+            if renpy.config.developer:
+                raise Exception("Invalid key specifier %s" % key)
+            else:
+                return "(False)"
 
         key = "_".join(part)
         
@@ -194,14 +201,14 @@ def run(var, *args, **kwargs):
         rv = None
 
         for i in var:
-            new_rv = i(*args)
+            new_rv = run(i, *args, **kwargs)
 
             if new_rv is not None:
                 rv = new_rv
 
         return rv
 
-    return var(*args)
+    return var(*args, **kwargs)
 
 def run_unhovered(var):
     """
@@ -213,10 +220,7 @@ def run_unhovered(var):
 
     if isinstance(var, (list, tuple)):
         for i in var:
-
-            f = getattr(i, "unhovered", None)
-            if f is not None:
-                f()
+            run_unhovered(i)
 
         return
 
@@ -647,7 +651,7 @@ class Button(renpy.display.layout.Window):
 def TextButton(text, style='button', text_style='button_text',
                clicked=None, **properties):
 
-    text = renpy.display.text.Text(text, style=text_style)
+    text = renpy.text.text.Text(text, style=text_style) #@UndefinedVariable
     return Button(text, style=style, clicked=clicked, **properties)
 
 class ImageButton(Button):
@@ -714,7 +718,7 @@ class HoveredProxy(object):
             return self.b()
     
                 
-class Input(renpy.display.text.Text):
+class Input(renpy.text.text.Text): #@UndefinedVariable
     """
     This is a Displayable that takes text as input.
     """
@@ -736,7 +740,7 @@ class Input(renpy.display.text.Text):
                  replaces=None,
                  **properties):
 
-        super(Input, self).__init__("", style=style, replaces=replaces, **properties)
+        super(Input, self).__init__("", style=style, replaces=replaces, substitute=False, **properties)
 
         self.content = unicode(default)
         self.length = length
@@ -750,7 +754,8 @@ class Input(renpy.display.text.Text):
 
         self.editable = True
 
-        caretprops = { 'color' : None}
+        caretprops = { 'color' : None }
+        
         for i in properties:
             if i.endswith("color"):
                 caretprops[i] = properties[i]
@@ -781,12 +786,24 @@ class Input(renpy.display.text.Text):
                 self.changed(content)
                 
         self.editable = editable
+
+        # Choose the caret.
+        caret = self.style.caret
+        if caret is None:
+            caret = self.caret
                                             
         if editable:
-            self.set_text([self.prefix, content.replace("{", "{{"), self.suffix, self.caret])
+            self.set_text([self.prefix, content.replace("{", "{{"), self.suffix, caret])
         else:
             self.set_text([self.prefix, content.replace("{", "{{"), self.suffix ])
 
+    # This is needed to ensure the caret updates properly.
+    def set_style_prefix(self, prefix, root):
+        if prefix != self.style.prefix:
+            self.update_text(self.content, self.editable)
+
+        super(Input, self).set_style_prefix(prefix, root)
+
     def enable(self):
         self.update_text(self.content, True)
 
@@ -848,7 +865,7 @@ class Adjustment(renpy.object.Object):
     
     """
     
-    def __init__(self, range=1, value=0, step=None, page=0, changed=None, adjustable=None, ranged=None):
+    def __init__(self, range=1, value=0, step=None, page=0, changed=None, adjustable=None, ranged=None): #@ReservedAssignment
         """
         The following parameters correspond to fields or properties on
         the adjustment object:
@@ -864,7 +881,7 @@ class Adjustment(renpy.object.Object):
             defaults to 1/10th of a page, if set. Otherwise, defaults
             to the 1/20th of the range.
 
-           This is used when scrolling a viewport with the mouse wheel.
+            This is used when scrolling a viewport with the mouse wheel.
             
         `page`
             The page size of the adjustment. If None, this is set
@@ -873,28 +890,28 @@ class Adjustment(renpy.object.Object):
 
             It's can be used when clicking on a scrollbar.
 
-         The following parameters control the behavior of the adjustment.
+        The following parameters control the behavior of the adjustment.
 
-         `adjustable`
-              If True, this adjustment can be changed by a bar. If False,
-              it can't.
+        `adjustable`
+             If True, this adjustment can be changed by a bar. If False,
+             it can't.
 
-              It defaults to being adjustable if a `changed` function
-              is given or if the adjustment is associated with a viewport,
-              and not adjustable otherwise.
+             It defaults to being adjustable if a `changed` function
+             is given or if the adjustment is associated with a viewport,
+             and not adjustable otherwise.
 
-         `changed`
-             This function is called with the new value when the value of
-             the adjustment changes.
+        `changed`
+            This function is called with the new value when the value of
+            the adjustment changes.
 
-         `ranged`
-             This function is called with the adjustment object when
-             the range of the adjustment is set by a viewport.
+        `ranged`
+            This function is called with the adjustment object when
+            the range of the adjustment is set by a viewport.
 
-         .. method:: change(value)
+        .. method:: change(value)
 
-             Changes the value of the adjustment to `value`, updating
-             any bars and viewports that use the adjustment.
+            Changes the value of the adjustment to `value`, updating
+            any bars and viewports that use the adjustment.
          """
 
 
@@ -928,7 +945,7 @@ class Adjustment(renpy.object.Object):
         if self.ranged:
             self.ranged(self)
         
-    range = property(get_range, set_range)
+    range = property(get_range, set_range) #@ReservedAssignment
 
     def get_page(self):
         if self._page is not None:
@@ -945,11 +962,11 @@ class Adjustment(renpy.object.Object):
         if self._step is not None:
             return self._step
 
-        if self._page is not None:
+        if self._page is not None and self.page > 0:
             return self._page / 10
 
         if isinstance(self._range, float):
-            return self._range / 20
+            return self._range / 10
         else:
             return 1
 
@@ -999,7 +1016,7 @@ class Bar(renpy.display.core.Displayable):
             self.value = None
             
     def __init__(self,
-                 range=None,
+                 range=None, #@ReservedAssignment
                  value=None,
                  width=None,
                  height=None,
@@ -1104,7 +1121,7 @@ class Bar(renpy.display.core.Displayable):
         # Store the width and height for the event function to use.
         self.width = width
         self.height = height
-        range = self.adjustment.range
+        range = self.adjustment.range #@ReservedAssignment
         value = self.adjustment.value
         page = self.adjustment.page
 
@@ -1233,7 +1250,7 @@ class Bar(renpy.display.core.Displayable):
         if self.hidden:
             return None
         
-        range = self.adjustment.range
+        range = self.adjustment.range #@ReservedAssignment
         old_value = self.adjustment.value
         value = old_value
 
@@ -1407,13 +1424,13 @@ class Timer(renpy.display.layout.Null):
         if not self.repeat:
             state.next_event = None
         else:
-            state.next_event = state.next_event + st
+            state.next_event = state.next_event + self.delay
             if state.next_event < st:
                 state.next_event = st + self.delay
 
             renpy.game.interface.timeout(state.next_event - st)
 
-        return self.function(*self.args, **self.kwargs)
+        return run(self.function, *self.args, **self.kwargs)
 
     
 class MouseArea(renpy.display.core.Displayable):
