@@ -1,4 +1,4 @@
-# Copyright 2004-2011 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2012 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -30,6 +30,17 @@ from renpy.display.layout import Container
 
 import renpy.display.accelerator
 
+# The null object that's used if we don't have a defined child.
+null = None 
+
+def get_null():
+    global null
+    
+    if null is None:
+        null = renpy.display.layout.Null()
+        
+    return null
+
 # Convert a position from cartesian to polar coordinates.
 def cartesian_to_polar(x, y, xaround, yaround):
     """
@@ -355,8 +366,10 @@ class Proxy(object):
     def __set__(self, instance, value):
         return setattr(instance.state, self.name, value)
     
-
 class Transform(Container):
+    """
+    Documented in sphinx, because we can't scan this object.
+    """
 
     __version__ = 5
     transform_event_responder = True
@@ -614,6 +627,9 @@ class Transform(Container):
     
     def _hide(self, st, at, kind):
 
+        if not self.child:
+            return None
+
         if not (self.hide_request or self.replaced_request):
             d = self.copy()
         else:
@@ -647,6 +663,7 @@ class Transform(Container):
         return None
         
     def set_child(self, child):
+        
         child = renpy.easy.displayable(child)
         
         self.child = child
@@ -677,7 +694,7 @@ class Transform(Container):
 
         # Use non-None elements of the child placement as defaults.
         child = self.child
-        if child and renpy.config.transform_uses_child_position:
+        if child is not None and renpy.config.transform_uses_child_position:
 
             pos = child.get_placement()
 
@@ -723,6 +740,10 @@ class Transform(Container):
 
         if child is None:
             child = self.child
+
+        # If we don't have a child for some reason, set it to null.            
+        if child is None:
+            child = get_null()
         
         rv = Transform(
             child=child,
@@ -752,6 +773,11 @@ class Transform(Container):
         return rv
 
     def update(self):
+        """
+        This should be called when a transform property field is updated outside
+        of the callback method, to ensure that the change takes effect.
+        """
+        
         renpy.display.render.invalidate(self)
 
     def parameterize(self, name, parameters):
