@@ -1,4 +1,4 @@
-# Copyright 2004-2011 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2012 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -24,6 +24,7 @@ import pygame
 import math
 import weakref
 import time
+import os
 
 from renpy.display.render import blit_lock, IDENTITY, BLIT, DISSOLVE, IMAGEDISSOLVE, PIXELLATE
 
@@ -233,6 +234,26 @@ class Clipper(object):
             
 clippers = [ Clipper() ]        
 
+def surface(w, h, alpha):
+    """
+    Creates a surface that shares a pixel format with the screen. The created
+    surface will 
+    """
+        
+    if alpha:
+        rv = pygame.Surface((w + 4, h + 4), pygame.SRCALPHA)
+    else:
+        rv = pygame.Surface((w + 4, h + 4), 0)
+        
+    return rv.subsurface((2, 2, w, h))
+
+def copy_surface(surf):
+    w, h = surf.get_size()
+    
+    rv = surface(w, h, True)
+    rv.blit(surf, (0, 0))
+    
+    return rv
 
 def draw_special(what, dest, x, y):
     """
@@ -248,14 +269,14 @@ def draw_special(what, dest, x, y):
 
     if w <= 0 or h <= 0:
         return
-    
-    if what.operation == DISSOLVE:
 
+    if what.operation == DISSOLVE:
+    
         bottom = what.children[0][0].render_to_texture(True)
         top = what.children[1][0].render_to_texture(True)
         
         if what.operation_alpha:
-            target = renpy.display.pgrender.surface((w, h), True)
+            target = surface(w, h, True)
         else:
             target = dest.subsurface((0, 0, w, h))
         
@@ -275,7 +296,7 @@ def draw_special(what, dest, x, y):
         top = what.children[2][0].render_to_texture(True)
 
         if what.operation_alpha:
-            target = renpy.display.pgrender.surface((w, h), True)
+            target = surface(w, h, True)
         else:
             target = dest.subsurface((0, 0, w, h))
 
@@ -593,12 +614,10 @@ def draw_transformed(dest, clip, what, xo, yo, alpha, forward, reverse):
 
             dest = dest.subsurface((x, y, width, height))
         
-        
-    if what.draw_func:
+    if what.draw_func or what.operation != BLIT:
         child = what.pygame_surface(True)
         draw_transformed(dest, clip, child, xo, yo, alpha, forward, reverse)
-        
-        # raise Exception("Using a draw_func on a transformed surface is not supported.")
+        return
 
     for child, cxo, cyo, _focus, _main in what.visible_children:
 
@@ -615,7 +634,7 @@ def draw_transformed(dest, clip, what, xo, yo, alpha, forward, reverse):
 
 
 
-def do_draw_screen(screen_render, full_redraw):
+def do_draw_screen(screen_render, full_redraw, swdraw):
     """
     Draws the render produced by render_screen to the screen.
     """
@@ -636,7 +655,7 @@ def do_draw_screen(screen_render, full_redraw):
 
     x, y, _w, _h = cliprect
 
-    dest = pygame.display.get_surface().subsurface(cliprect)
+    dest = swdraw.window.subsurface(cliprect)
     draw(dest, None, screen_render, -x, -y, True)
 
     return updates
@@ -648,6 +667,11 @@ class SWDraw(object):
     """
 
     def __init__(self):
+        self.display_info = None
+
+        self.reset()
+
+    def reset(self):
         
         # Should we draw the screen?
         self.suppressed_blit = False
@@ -676,18 +700,28 @@ class SWDraw(object):
 
         pygame.display.init()
         renpy.display.interface.post_init()
-        
-        self.display_info = pygame.display.Info()
 
+        if self.display_info is None:       
+            self.display_info = pygame.display.Info()
+
+        # The scale factor we use for this display.
+        self.scale_factor = 1.0
         
+        # Should we scale fast, or scale good-looking?
+        self.scale_fast = "RENPY_SCALE_FAST" in os.environ
         
+        # The screen returned to us from pygame.
+        self.screen = None
+        
+        # The window that we render into, if not the screen. This has a 
+        # 1px border around it iff we're scaling.
+        self.window = None
+                
     def set_mode(self, virtual_size, physical_size, fullscreen):
 
         # Reset before resize.
         renpy.display.interface.kill_textures_and_surfaces()
-
-        # Clear out the old fonts.
-        renpy.display.font.font_cache.clear()
+        self.reset()
 
         width, height = virtual_size
 
@@ -700,16 +734,9 @@ class SWDraw(object):
             screen_width -= 102
 
         scale_factor = min(1.0 * screen_width / width, 1.0 * screen_height / height, 1.0)
-
-        # Pick an even scale factor, which seems to help w/ visual artifacts.)
-        for i in range(100, 45, -5):
-            i /= 100.0
-            
-            if scale_factor >= i:
-                scale_factor = i
-                break
-        
-        renpy.display.scale.init(scale_factor)        
+        if "RENPY_SCALE_FACTOR" in os.environ:
+            scale_factor = float(os.environ["RENPY_SCALE_FACTOR"])
+        self.scale_factor = scale_factor
 
         # Figure out the fullscreen info.
         if fullscreen:
@@ -719,25 +746,32 @@ class SWDraw(object):
               
         # If a window exists of the right size and flags, use it. Otherwise,
         # make our own window.
-        old_window = pygame.display.get_surface()
+        old_screen = pygame.display.get_surface()
 
-        if ((old_window is not None) and 
-            (old_window.get_size() == (width, height)) and
-            (old_window.get_flags() & pygame.FULLSCREEN == fsflag)):
+        scaled_width = int(width * scale_factor)
+        scaled_height = int(height * scale_factor)
+
+        if ((old_screen is not None) and 
+            (old_screen.get_size() == (scaled_width, scaled_height)) and
+            (old_screen.get_flags() & pygame.FULLSCREEN == fsflag)):
             
-            self.window = old_window
+            self.screen = old_screen
                     
         else:
-            self.window = renpy.display.pgrender.set_mode(
-                (width, height),
-                fsflag,
-                32)
+            self.screen = pygame.display.set_mode((scaled_width, scaled_height), fsflag, 32)
+            
+        if scale_factor != 1.0:
+            self.window = surface(width, height, True)
+        else:
+            self.window = self.screen
+            
+        renpy.display.pgrender.set_rgba_masks()
             
         # Should we redraw the screen from scratch?
         self.full_redraw = True
 
         # The surface used to display fullscreen video.
-        self.fullscreen_surface = renpy.display.scale.real(self.window)
+        self.fullscreen_surface = self.screen
 
         # Reset this on a mode change.
         self.mouse_location = None
@@ -746,7 +780,6 @@ class SWDraw(object):
         self.mouse_info = None
 
         return True
-        
 
     # private
     def show_mouse(self, pos, info):
@@ -766,10 +799,10 @@ class SWDraw(object):
         by = my - myo
 
         self.mouse_backing_pos = (bx, by)
-        self.mouse_backing = renpy.display.pgrender.surface((mw, mh), False)
+        self.mouse_backing = surface(mw, mh, False)
         self.mouse_backing.blit(self.window, (0, 0), (bx, by, mw, mh))
 
-        self.window.blit(tex, (bx, by))
+        self.screen.blit(tex, (bx, by))
 
         return bx, by, mw, mh
 
@@ -780,7 +813,7 @@ class SWDraw(object):
         """
         
         size = self.mouse_backing.get_size()
-        self.window.blit(self.mouse_backing, self.mouse_backing_pos)
+        self.screen.blit(self.mouse_backing, self.mouse_backing_pos)
 
         rv = self.mouse_backing_pos + size
 
@@ -840,10 +873,20 @@ class SWDraw(object):
             
     def mouse_event(self, ev):        
         x, y = getattr(ev, 'pos', pygame.mouse.get_pos())
+        
+        x /= self.scale_factor
+        y /= self.scale_factor
+        
         return x, y
         
     def get_mouse_pos(self):
-        return pygame.mouse.get_pos()
+        x, y = pygame.mouse.get_pos()
+        
+        x /= self.scale_factor
+        y /= self.scale_factor
+        
+        return x, y
+    
     
     def screenshot(self, surftree, fullscreen_video):
         """
@@ -900,15 +943,27 @@ class SWDraw(object):
 
             updates.extend(self.draw_mouse(False))
 
-            damage = do_draw_screen(surftree, self.full_redraw)
+            damage = do_draw_screen(surftree, self.full_redraw, self)
 
             if damage:
                 updates.extend(damage)
 
             self.full_redraw = False
 
-            updates.extend(self.draw_mouse(True))
-            pygame.display.update(updates)
+            if self.window is self.screen:
+
+                updates.extend(self.draw_mouse(True))
+                pygame.display.update(updates)
+
+            else:
+                
+                if self.scale_fast:
+                    pygame.transform.scale(self.window, self.screen.get_size(), self.screen)
+                else:
+                    renpy.display.scale.smoothscale(self.window, self.screen.get_size(), self.screen)
+                
+                self.draw_mouse(True)                
+                pygame.display.flip()
             
         else:
             pygame.display.flip()
@@ -918,7 +973,8 @@ class SWDraw(object):
 
         
     def render_to_texture(self, render, alpha):
-        rv = renpy.display.pgrender.surface((render.width, render.height), alpha)
+  
+        rv = surface(render.width, render.height, alpha)
         draw(rv, None, render, 0, 0, False)
 
         return rv
@@ -941,10 +997,15 @@ class SWDraw(object):
                     return True
 
             else:
+                cx = int(cx)
+                cy = int(cy)
+                
                 cw, ch = child.get_size()
                 if cx >= cw or cy >= ch:
                     return False
 
+                
+
                 if not child.get_masks()[3] or child.get_at((cx, cy))[3]:
                     return True
 
@@ -970,6 +1031,9 @@ class SWDraw(object):
         is in the RLE cache.
         """
 
+        surf = copy_surface(surf)
+        self.mutated_surface(surf)
+
         if transient:
             return surf
 
@@ -977,7 +1041,7 @@ class SWDraw(object):
             return surf
 
         if surf not in rle_cache:
-            rle_surf = renpy.display.pgrender.copy_surface(surf)
+            rle_surf = copy_surface(surf)
             rle_surf.set_alpha(255, pygame.RLEACCEL)
             self.mutated_surface(rle_surf)
 
@@ -985,6 +1049,20 @@ class SWDraw(object):
         
         return surf
 
+    def solid_texture(self, w, h, color):
+        """
+        Creates a texture filled to the edges with color.
+        """
+        
+        surf = surface(w + 4, h + 4, True)
+        surf.fill(color)
+        self.mutated_surface(surf)
+
+        surf = surf.subsurface((2, 2, w, h))
+        
+        self.mutated_surface(surf)
+        return surf
+        
         
     def free_memory(self):
         """
@@ -1002,7 +1080,7 @@ class SWDraw(object):
         
         return
         
-    def quit(self):
+    def quit(self): #@ReservedAssignment
         """
         Shuts down the drawing system.
         """
