@@ -1,4 +1,4 @@
-# Copyright 2004-2011 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2012 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -49,10 +49,6 @@ class Transition(renpy.display.core.Displayable):
     def visit(self):
         return [ self.new_widget, self.old_widget ] # E1101
 
-    def per_interact(self):
-        self.new_widget.per_interact()
-        self.old_widget.per_interact()
-    
 
 def null_render(d, width, height, st, at):
 
@@ -151,13 +147,9 @@ class MultipleTransition(Transition):
 
     def visit(self):
         return [ i for i in self.screens if isinstance(i, renpy.display.core.Displayable)] + self.transitions
-
-    def per_interact(self):
-        for i in self.visit():
-            i.per_interact()
     
     def event(self, ev, x, y, st):
-        
+
         if self.events or ev.type == renpy.display.core.TIMEEVENT:
             return self.transitions[-1].event(ev, x, y, st)
         else:
@@ -206,7 +198,7 @@ def Fade(out_time,
 
     """
     :doc: transition function
-    :args: (out_time, hold_time, old_time, color="#000")
+    :args: (out_time, hold_time, in_time, color="#000")
     :name: Fade
     
     Returns a transition that takes `out_time` seconds to fade to 
@@ -314,18 +306,23 @@ class Pixellate(Transition):
 class Dissolve(Transition):
     """
     :doc: transition function
-    :args: (time, alpha=False)
+    :args: (time, alpha=False, time_warp=None)
     :name: Dissolve
     
     Returns a transition that dissolves from the old scene to the new scene.
     
     `time`
         The time the dissolve will take.
-        
+
     `alpha`
         If true, the dissolve will alpha-composite the the result of the 
         transition with the screen. If false, the result of the transition
         will replace the screen, which is more efficient.
+
+    `time_warp`
+        A function that adjusts the timeline. If not None, this should be a 
+        function that takes a fractional time between 0.0 and 1.0, and returns 
+        a number in the same range. 
     """
 
     __version__ = 1
@@ -334,7 +331,9 @@ class Dissolve(Transition):
         if version < 1:
             self.alpha = False
     
-    def __init__(self, time, old_widget=None, new_widget=None, alpha=False, **properties):
+    time_warp = None
+    
+    def __init__(self, time, old_widget=None, new_widget=None, alpha=False, time_warp=None, **properties):
         super(Dissolve, self).__init__(time, **properties)
 
         self.time = time
@@ -342,6 +341,7 @@ class Dissolve(Transition):
         self.new_widget = new_widget
         self.events = False
         self.alpha = alpha
+        self.time_warp = time_warp
 
 
     def render(self, width, height, st, at):
@@ -355,6 +355,9 @@ class Dissolve(Transition):
 
         complete = min(1.0, st / self.time)
 
+        if self.time_warp is not None:
+            complete = self.time_warp(complete)
+
         bottom = render(self.old_widget, width, height, st, at)
         top = render(self.new_widget, width, height, st, at)
         
@@ -378,7 +381,7 @@ class Dissolve(Transition):
 class ImageDissolve(Transition):
     """
     :doc: transition function
-    :args: (image, time, ramplen=8, reverse=False, alpha=True)
+    :args: (image, time, ramplen=8, reverse=False, alpha=True, time_warp=None)
     :name: ImageDissolve
 
     Returns a transition that dissolves the old scene into the new scene, using
@@ -406,7 +409,12 @@ class ImageDissolve(Transition):
         If true, the dissolve will alpha-composite the the result of the 
         transition with the screen. If false, the result of the transition
         will replace the screen, which is more efficient.
-        
+
+    `time_warp`
+        A function that adjusts the timeline. If not None, this should be a 
+        function that takes a fractional time between 0.0 and 1.0, and returns 
+        a number in the same range.         
+ 
     ::
     
         define circirisout = ImageDissolve("circiris.png", 1.0)
@@ -420,6 +428,8 @@ class ImageDissolve(Transition):
         if version < 1:
             self.alpha = False
     
+    time_warp = None
+    
     def __init__(
         self,
         image,
@@ -431,6 +441,7 @@ class ImageDissolve(Transition):
         alpha=False,
         old_widget=None,
         new_widget=None,
+        time_warp=None,
         **properties):
 
         # ramptype and ramp are now unused, but are kept for compatbility with
@@ -442,6 +453,7 @@ class ImageDissolve(Transition):
         self.new_widget = new_widget
         self.events = False
         self.alpha = alpha
+        self.time_warp = time_warp
         
         if not reverse:
 
@@ -492,9 +504,14 @@ class ImageDissolve(Transition):
 
         rv = renpy.display.render.Render(width, height, opaque=not self.alpha)
 
+        complete = st / self.delay
+        
+        if self.time_warp is not None:
+            complete = self.time_warp(complete)
+
         rv.operation = renpy.display.render.IMAGEDISSOLVE
         rv.operation_alpha = self.alpha
-        rv.operation_complete = st / self.delay
+        rv.operation_complete = complete
         rv.operation_parameter = self.ramplen
 
         rv.blit(image, (0, 0), focus=False, main=False)
@@ -529,7 +546,7 @@ class AlphaDissolve(Transition):
     `reverse`
         If true, the alpha channel is reversed. Opaque areas are taken
         from the old image, while transparent areas are taken from the
-        new image.
+        new image. 
      """
 
     def __init__(
