@@ -1,4 +1,4 @@
-# Copyright 2004-2011 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2012 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -34,7 +34,6 @@ import sets
 
 import renpy.audio
 
-
 ##### Code that computes reachable objects, which is used to filter
 ##### the rollback list before rollback or serialization.
 
@@ -45,6 +44,7 @@ class NoRollback(object):
     
     pass
 
+
 # parents = [ ]
 
 def reached(obj, reachable, wait):
@@ -178,6 +178,8 @@ def set_filename(filename, offset, tree):
         worklist.extend(node.getChildNodes())
 
 
+unicode_re = re.compile(ur'[\u0080-\uffff]')
+
 def unicode_sub(m):
     """
     If the string s contains a unicode character, make it into a
@@ -186,7 +188,7 @@ def unicode_sub(m):
 
     s = m.group(0)
 
-    if "\\u" not in s:
+    if not unicode_re.search(s):
         return s
     
     prefix = m.group(1)
@@ -197,17 +199,16 @@ def unicode_sub(m):
         prefix = 'u' + prefix
 
     rv = prefix + sep + body + sep
+
     return rv
     
 
 string_re = re.compile(r'([uU]?[rR]?)("""|"|\'\'\'|\')((\\.|.)*?)\2')
 
-def escape_unicode(s):
-    s = s.encode("raw_unicode_escape")
-    
-    if "\\u" in s:
+def escape_unicode(s):    
+    if unicode_re.search(s):
         s = string_re.sub(unicode_sub, s)
-
+    
     return s
 
 def py_compile(source, mode, filename='<none>', lineno=1):
@@ -232,6 +233,7 @@ def py_compile(source, mode, filename='<none>', lineno=1):
         filename = source.filename
         lineno = source.linenumber
 
+    source = unicode(source)
     source = source.replace("\r", "")
     source = escape_unicode(source)
         
@@ -432,6 +434,7 @@ class RevertableObject(object):
         self.__dict__.clear()
         self.__dict__.update(old)
 
+
 ##### An object that handles deterministic randomness, or something.
 
 class DetRandom(random.Random):
@@ -650,12 +653,7 @@ class RollbackLog(renpy.object.Object):
 
         # If the log is too long, try pruning it to a label.
         if len(self.log) > renpy.config.rollback_length:
-            rb = self.log[-renpy.config.rollback_length]
-
-            # Checks to see if this is a real name, rather than a
-            # tuple.
-            if isinstance(rb.context.current, basestring):
-                self.log = self.log[-renpy.config.rollback_length:]
+            self.log = self.log[-renpy.config.rollback_length:]
 
         self.current = Rollback()
         self.log.append(self.current)
@@ -853,7 +851,7 @@ class RollbackLog(renpy.object.Object):
 
         # If we have exceeded the rollback limit, and don't have force,
         # give up.
-        if not self.rollback_limit > 0 and not force:
+        if checkpoints and not self.rollback_limit > 0 and not force:
             return
 
         self.purge_unreachable(self.get_roots())
@@ -960,16 +958,16 @@ class RollbackLog(renpy.object.Object):
 
         # We never make it this far.
 
-def py_exec_bytecode(bytecode, hide=False, globals=None, locals=None):
+def py_exec_bytecode(bytecode, hide=False, globals=None, locals=None): #@ReservedAssignment
 
     if hide:
-        locals = { }
+        locals = { } #@ReservedAssignment
 
     if globals is None:
-        globals = renpy.store.__dict__ #@UndefinedVariable
+        globals = renpy.store.__dict__ #@UndefinedVariable @ReservedAssignment
 
     if locals is None:
-        locals = globals
+        locals = globals #@ReservedAssignment
 
     exec bytecode in globals, locals
 
@@ -980,33 +978,33 @@ def py_exec(source, hide=False, store=None):
         store = vars(renpy.store)
 
     if hide:
-        locals = { }
+        locals = { } #@ReservedAssignment
     else:
-        locals = store
+        locals = store #@ReservedAssignment
 
     exec py_compile(source, 'exec') in store, locals
 
 
-def py_eval_bytecode(bytecode, globals=None, locals=None):
+def py_eval_bytecode(bytecode, globals=None, locals=None): #@ReservedAssignment
 
     if globals is None:
-        globals = renpy.store.__dict__ #@UndefinedVariable
+        globals = renpy.store.__dict__ #@UndefinedVariable @ReservedAssignment
 
     if locals is None:
-        locals = globals
+        locals = globals #@ReservedAssignment
 
     return eval(bytecode, globals, locals)
 
 
-def py_eval(source, globals=None, locals=None):
+def py_eval(source, globals=None, locals=None): #@ReservedAssignment
     
     # source = source.strip()
 
     if globals is None:
-        globals = renpy.store.__dict__ #@UndefinedVariable
+        globals = renpy.store.__dict__ #@UndefinedVariable @ReservedAssignment
 
     if locals is None:
-        locals = globals
+        locals = globals #@ReservedAssignment
     
     return eval(py_compile(source, 'eval'), globals, locals)
 
@@ -1041,5 +1039,3 @@ def method_unpickle(obj, name):
 import copy_reg
 import types
 copy_reg.pickle(types.MethodType, method_pickle, method_unpickle)
-
-
