@@ -1,4 +1,4 @@
-# Copyright 2004-2011 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2012 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -270,7 +270,7 @@ class FunctionStatementParser(Parser):
     This is responsible for parsing function statements.
     """
 
-    def __init__(self, name, function, nchildren=0, unevaluated=False):
+    def __init__(self, name, function, nchildren=0, unevaluated=False, scope=False):
 
         super(FunctionStatementParser, self).__init__(name)
         
@@ -291,6 +291,8 @@ class FunctionStatementParser(Parser):
         if nchildren != 0:
             childbearing_statements.append(self)
 
+        self.scope = scope
+
     def parse_layout(self, l, name):
         return self.parse(l, name, True)
             
@@ -446,6 +448,9 @@ class FunctionStatementParser(Parser):
         if "id" not in seen_keywords:
             call_node.keywords.append(ast.keyword(arg="id", value=self.parse_eval(name, lineno)))
             
+        if "scope" not in seen_keywords and self.scope:
+            call_node.keywords.append(ast.keyword(arg="scope", value=self.parse_eval("_scope", lineno)))
+            
         return rv
 
         
@@ -496,16 +501,21 @@ text_properties = [ Style(i) for i in [
         "hyperlink_functions",
         "italic",
         "justify",
+        "kerning",
         "language",
         "layout",
+        "line_leading",
         "line_spacing",
         "minwidth",
         "min_width",
-        "outlines",
+        "newline_indent",
+        "outlines",        
         "rest_indent",
+        "ruby_style",
         "slow_cps",
         "slow_cps_multiplier",
         "slow_abortable",
+        "strikethrough",
         "text_align",
         "text_y_fudge",
         "underline",
@@ -595,9 +605,12 @@ add(ui_properties)
 add(position_properties)
 
 
-FunctionStatementParser("text", "ui.text", 0)
+FunctionStatementParser("text", "ui.text", 0, scope=True)
 Positional("text")
 Keyword("slow")
+Keyword("slow_done")
+Keyword("substitute")
+Keyword("scope")
 add(ui_properties)
 add(position_properties)
 add(text_properties)
@@ -695,19 +708,21 @@ add(position_properties)
 add(window_properties)
 add(button_properties)
 
-FunctionStatementParser("textbutton", "ui.textbutton", 0)
+FunctionStatementParser("textbutton", "ui.textbutton", 0, scope=True)
 Positional("label")
 Keyword("action")
 Keyword("clicked")
 Keyword("hovered")
 Keyword("unhovered")
 Keyword("text_style")
+Keyword("substitute")
+Keyword("scope")
 add(ui_properties)
 add(position_properties)
 add(window_properties)
 add(button_properties)
 
-FunctionStatementParser("label", "ui.label", 0)
+FunctionStatementParser("label", "ui.label", 0, scope=True)
 Positional("label")
 Keyword("text_style")
 add(ui_properties)
@@ -968,7 +983,7 @@ class ForParser(Parser):
             if l.match(r"\("):
                 p = self.parse_tuple_pattern(l)
             else:
-                p = l.name().encode("ascii")
+                p = l.name().encode("utf-8")
 
             if not p:
                 break
